@@ -214,6 +214,16 @@ userconfig::userconfig(const std::string& name,
             "is not set). Discarded reads are replaced with a single 'N' with "
             "Phred score 0 [default: %default].");
 
+  argparser["--mask-degenerate-bases"] =
+        new argparse::flag(nullptr,
+             "Mask degenerate/ambiguous bases (B/D/H/K/M/N/R/S/V/W/Y) in "
+            "input by replacing them with an 'N'; if this option is not used, "
+            "AdapterRemoval will abort upon encountering degenerate bases.");
+  argparser["--convert-uracils"] =
+        new argparse::flag(nullptr,
+            "Convert uracils (U) to thymine (T) in input reads; if this option "
+            "is not used, AdapterRemoval will abort upon encountering uracils.");
+
     argparser.add_header("OUTPUT FILES:");
     argparser["--basename"] =
         new argparse::any(&basename, "BASENAME",
@@ -388,7 +398,7 @@ userconfig::userconfig(const std::string& name,
             "one of two different overlapping bases if these have the same "
             "quality (otherwise it picks the highest quality base). With "
             "--collapse-deterministic, AdapterRemoval will instead set such "
-            "bases to N. Setting this option also sets--collapse "
+            "bases to N. Setting this option also sets --collapse "
             "[default: %default].");
     argparser["--collapse-conservatively"] =
         new argparse::flag(&collapse_conservatively,
@@ -422,7 +432,8 @@ userconfig::userconfig(const std::string& name,
     argparser["--barcode-mm"] =
         new argparse::knob(&barcode_mm, "N",
             "Maximum number of mismatches allowed when counting mismatches in "
-            "both the mate 1 and the mate 2 barcode for paired reads.");
+            "both the mate 1 and the mate 2 barcode for paired reads "
+            "[default: %default].");
     argparser["--barcode-mm-r1"] =
         new argparse::knob(&barcode_mm_r1, "N",
             "Maximum number of mismatches allowed for the mate 1 barcode; "
@@ -462,6 +473,9 @@ argparse::parse_result userconfig::parse_args(int argc, char *argv[])
         return argparse::parse_result::error;
     }
 
+    quality_input_fmt->mask_degenerate_bases(argparser.is_set("--mask-degenerate-bases"));
+    quality_input_fmt->convert_uracils(argparser.is_set("--convert-uracils"));
+
     if (argparser.is_set("--qualitybase-output")) {
         quality_output_fmt = select_encoding("--qualitybase-out", quality_output_base, quality_max);
         if (!quality_output_fmt.get()) {
